#!/usr/bin/perl -w
use strict;
use Getopt::Std;
use File::Basename;
use Math::Trig;
use Symbol ();

########################################
# Version 0.1.0, R.C., 15.01.2010  
# first public version
#########################################


my($version) = "0.1.0";

my($options) = "$0 [options] <file>
   Author: Raymond Choc, ptextsplit(at)melibokus-biker.de
   Extract the informations of an CompeGPS CLAY file 
   and write it to an configuration file.
   Example: clayExcrator.pl *.clay
            read all clay's and write the clay configuration to 
            *.clay.cfg
   Example: clayExcrator.pl -d my *.clay
            read all clay's and write the clay configuration to 
            my*.clay.cfg
   Example: clayExcrator.pl -d basic DACH08Q4.clay
            read all clay and write the clay configuration to 
            basic.clay.cfg
   Options:
     -v version info
     -q be quiet
     -h print this help
     -d write exported clay configuration to this file (optional)
     \n";

my($bequiet) = 0;
my($ifname,$ofname,$prefixFilename);
my($sortOrder) = -1;

die("Option error!\n$options")
    unless getopts('vqhd:');

if (defined($Getopt::Std::opt_v)) {
    print STDERR "$0 Version $version\n";
		$Getopt::Std::opt_v++;
    exit;
}if (defined($Getopt::Std::opt_h)) {
  print STDERR ($options);
	$Getopt::Std::opt_h++;
  exit 0;
}
if (defined($Getopt::Std::opt_d)) {
	$prefixFilename = $Getopt::Std::opt_d;
}
if (defined($Getopt::Std::opt_q)) {
	$bequiet = 1;
	$Getopt::Std::opt_q++;
}

if (!defined($ARGV[0])) {
	die("No file to split!\n$options");
}

my(@fileList);
foreach (@ARGV){
	if( -d $_){
		$_ .="\\*.clay";
		print "$_\n";
	}
	my($filename,$path) = fileparse($_);
	chdir($path);
	$filename =~ s/ /?/g;
	push @fileList, map {$path.$_} glob $filename;
	
	my $useAsPrefix = (@fileList > 1 && defined $prefixFilename)?1:0;
	foreach $ifname (@fileList){
		my %clayConfig= readClay($ifname);
		$ofname = clayConfigFileName($ifname, $prefixFilename, $useAsPrefix);
		printf "FILENAME %s\n", $ofname;
		writeClayConfig($ofname, %clayConfig);
	}
}

sub clayConfigFileName{
	my($ifname, $prefixFilename, $useAsPrefix) = @_;
	my($name,$path,$suffix) = fileparse($ifname,qw{\.\w*$});
	$ofname = $path;
	if($useAsPrefix){
		$ofname .=  $prefixFilename.$name;
	}elsif(defined $prefixFilename){
		$ofname .= $prefixFilename;
	}else{
		$ofname .= $name;
	}
	$ofname .= $suffix.".cgf";
}

sub readClay{
	my($ifname) =@_;
	my(%clayConfig,%layers,%colors,%lineTyps,%routingTyps,%areaPatterns);
	unless (open(INPUT,"<$ifname")) {
	    die "Cannot open '$ifname' for reading";
	}

	my($es)=0;
	my($readHeader)=0;
	my($id);
	my $i;
	my @languages;
	$languages[0]='ES';
	$clayConfig{languages} = \@languages;
	while (<INPUT>){
#		print $_;
		chomp;
  	if (/^\s*$/){
			next;
  	}
		if(/^\s*<Style>.*$/){
			while(<INPUT>){
#				print $_;
				chomp;
				if(/^\s*Languages\=([\w\,]+).*$/){
					#     Languages=EN,ES,FR,DE
#					print $1."\n";
					@languages= split /\,/, $1;
					$clayConfig{languages} = \@languages;
					foreach(@languages){
#						print $languages[$es]."\n";
						if( $languages[$es] eq 'ES'){
							last;
						}
						$es++;
					}
				}elsif(/^\s*<\/Style>/){
					last;
				}
			}
		}
		if(/^\s*Layer\d+\=\d+,([^\,]+),([\d\-]+),([\d\-]+),(\d),([\d\.\-]+),([\d\.\-]+),([\d\-]+),([\d\-]+),([\d\.\-]+),([\d\.\-]+),([^\,]*),([\d\-]+),(\d),([\d\.\-]+),([\d\-]+),([\d\-]+),(\d+),(\d),([^,]*),(\d),([\d\.\-]+),([\d\-]+),([\d\-]+).*$/){
# Layer0=7168,Built-up Area | Zona Urbana | Zone Urbaine | Ortslage,11587327,11587327,1,2000.0000,-1.0000,-1,1,0.0000,0.0000,,0,0,0.0000,210,1,286494,1,FFFFFFFFFFFFFFFF,4,0.0000,6526900,6526900
#			print $1."\n";
			$id=$1;
			my @identifier;
			$i++;
			my $layer ={borderColor => $2,         
							fillColor => $3,               
							showLayer => $4,               
							upperZoomVisibility => $5,     
							transparency => $6,            
							areaPatternID => $7,           
							graphicLineWidth => $8,        
							metricLineWidth => $9,         
							lowerZoomVisibility => $10,    
							iconFileName => $11,           
							linePatternID => $12,          
							routingType => $13,            
							speed => $14,                  
							TypeID => $15,                 
							drawLabels => $16,             
							featureCount => $17,           
							expandSubLayers => $18,        
							firstPolylineID => $19,        
							drawPolygonsOverRasters => $20,
							elevation => $21,              
							nightBorderColor => $22,       
							nightFillColor => $23,         
			};
			if($id =~ /^.+\|.+$/){
				@identifier = split /\|/, $id;
				#			print $identifier[$es]." $es\n";
				foreach (@identifier){
					s/^\s*//;
					s/\s*$//;
				}
				$id=$identifier[$es];
			}else{
				$id =~ s/^\s*//;
				$id =~ s/\s*$//;
				$identifier[0] = $id;
			}
			$layer->{identifier} = \@identifier;
			$layers{$id}=$layer;
			foreach my $color ($layer->{borderColor},$layer->{fillColor},$layer->{nightBorderColor},$layer->{nightFillColor}){
#				printf "COLOR: %s, ",$color;
				if(!exists $colors{$color}){
					$colors{$color} = {value => $color};
				}
			}
			if(!exists $lineTyps{$layer->{linePatternID}}){
				$lineTyps{$layer->{linePatternID}} = {value => $layer->{linePatternID}};
			}
			if(!exists $areaPatterns{$layer->{areaPatternID}}){
				$areaPatterns{$layer->{areaPatternID}} = {value => $layer->{areaPatternID}};
			}
			if(!exists $routingTyps{$layer->{routingType}}){
				$routingTyps{$layer->{routingType}} = {value => $layer->{routingType}};
			}
		}else{
#			print $_."\n";
		}
	}
	$clayConfig{layers}=\%layers;
	$clayConfig{colors}=\%colors;
	$clayConfig{lineTyps}=\%lineTyps;
	$clayConfig{areaPatterns}=\%areaPatterns;
	$clayConfig{routingTyps}=\%routingTyps;
	close INPUT;
#	print $i .' '.(keys %layers)."\n";
	%clayConfig;
}

sub writeClayConfig{
	my($ofname, %clayConfig)=@_;
	
	unless (open(OUT,">$ofname")) {
    die "Cannot open '$ofname' for writing";
	}
	my $i;
	print OUT "[LANGUAGES]\n";
	foreach (@{$clayConfig{languages}}){
		printf OUT "%s\n", $_;
#		printf OUT "%s\n", 'Languages', join(',', @{$clayConfig{languages}});
	}
	print OUT "\n\n[COLORS]\n";
	print OUT "# replace the generic names (COLOR1,...) for colors with real color names!\n";
	$i=1;
	foreach my $color (sort {$a <=> $b} keys %{$clayConfig{colors}}){
		$clayConfig{colors}->{$color}->{id}='COLOR_'.$i++;
		printf OUT "%s = %s\n", $clayConfig{colors}->{$color}->{id},$clayConfig{colors}->{$color}->{value};
	}
	print OUT "\n\n[LINETYPES]\n";
	foreach my $id (sort {$a <=> $b} keys %{$clayConfig{lineTyps}}){
		$clayConfig{lineTyps}->{$id}->{id}='LINETYPE_'.$id;
		printf OUT "%s = %d\n", $clayConfig{lineTyps}->{$id}->{id}, $clayConfig{lineTyps}->{$id}->{value};
	}
	print OUT "\n\n[AREAPATTERNS]\n";
	foreach my $id (sort {$a <=> $b} keys %{$clayConfig{areaPatterns}}){
		$clayConfig{areaPatterns}->{$id}->{id}='AREAPATTERN_'.$id;
		printf OUT "%s = %d\n", $clayConfig{areaPatterns}->{$id}->{id},$clayConfig{areaPatterns}->{$id}->{value};
	}
	print OUT "\n\n[ROUTINGTYPES]\n";
	foreach my $id (sort {$a <=> $b} keys %{$clayConfig{routingTyps}}){
		$clayConfig{routingTyps}->{$id}->{id}='ROUTINGTYPE_'.$id;
		printf OUT "%s = %d\n", $clayConfig{routingTyps}->{$id}->{id},$clayConfig{routingTyps}->{$id}->{value};
	}
	
	print OUT "\n\n########################################################################\n";
	print OUT "[LAYERS]\n";
	my %layers = %{$clayConfig{layers}};
	printf "%s\n",scalar %layers;
	foreach my $id (sort keys %layers){
		printf OUT "\n[%s]\n",$id;
#		printf OUT "identifier = %s\n",join '|', @{$layers{$id}->{identifier}};
		my $i=0;
		foreach (@{$layers{$id}->{identifier}}){
			printf OUT "identifier.%s = %s\n", @{$clayConfig{languages}}[$i++] ,$_;
		}
		printf OUT "showLayer = %d\n",$layers{$id}->{showLayer};
		printf OUT "borderColor = %s\n",$clayConfig{colors}->{$layers{$id}->{borderColor}}->{id};
		printf OUT "fillColor = %s\n",$clayConfig{colors}->{$layers{$id}->{fillColor}}->{id};
		printf OUT "nightBorderColor = %s\n",$clayConfig{colors}->{$layers{$id}->{nightBorderColor}}->{id};
		printf OUT "nightFillColor = %s\n",$clayConfig{colors}->{$layers{$id}->{nightFillColor}}->{id};
		printf OUT "linePatternID = %s\n",$clayConfig{lineTyps}->{$layers{$id}->{linePatternID}}->{id};
		printf OUT "graphicLineWidth = %d\n",$layers{$id}->{graphicLineWidth};
		printf OUT "metricLineWidth = %f\n",$layers{$id}->{metricLineWidth};
		printf OUT "areaPatternID = %s\n",$clayConfig{areaPatterns}->{$layers{$id}->{areaPatternID}}->{id};
		printf OUT "transparency = %d\n",$layers{$id}->{transparency};
		printf OUT "drawPolygonsOverRasters = %d\n",$layers{$id}->{drawPolygonsOverRasters};
		printf OUT "lowerZoomVisibility = %f\n",$layers{$id}->{lowerZoomVisibility};
		printf OUT "upperZoomVisibility = %f\n",$layers{$id}->{upperZoomVisibility};
		printf OUT "iconFileName = %s\n",$layers{$id}->{iconFileName};
		printf OUT "drawLabels = %d\n",$layers{$id}->{drawLabels};
		printf OUT "elevation = %f\n",$layers{$id}->{elevation};
		printf OUT "expandSubLayers = %d\n",$layers{$id}->{expandSubLayers};
		printf OUT "TypeID = %d\n",$layers{$id}->{TypeID};
		printf OUT "firstPolylineID = %s\n",$layers{$id}->{firstPolylineID};
		printf OUT "routingType = %s\n",$clayConfig{routingTyps}->{$layers{$id}->{routingType}}->{id};
		printf OUT "speed = %d\n",$layers{$id}->{speed};
	}

	close OUT;
}

